package particles

import("project-particles/config"
        "testing")


// Cette function test si NewSystem génére 0 particule

func TestNew0(t*testing.T){

  config.Get("../configtest.json")
  config.General.InitNumParticles = 0
  S := NewSystem()

  if len(S.Content) != 0{
    t.Fail()
  }
}

// Cette function test si NewSystem génére 10 particule

func TestNew10(t*testing.T){

  config.Get("../configtest.json")
  config.General.InitNumParticles = 10
  S := NewSystem()

  if len(S.Content) != 10{
    t.Fail()
  }
}

// Cette fonction test si NewSystem génére 10000 particule

func TestNew10000(t*testing.T){

  config.Get("../configtest.json")
  config.General.InitNumParticles = 10000
  S := NewSystem()

  if len(S.Content) != 10000{
    t.Fail()
  }
}

// Cette fonction test si la méthode Add génére 1 particule à chaque appel de la
// fonction Update

func TestAdd1(t*testing.T){

  config.Get("../configtest.json")
  config.General.InitNumParticles = 0
  config.General.SpawnRate = 1

  S := NewSystem()

  for i:=0; i< 60;i++{
    S.Add()
  }

  if len(S.Content) != 60{
    t.Fail()
  }
}

// Cette fonction test si Update génére 1 particule, un appel sur deux avec la
// méthode Add.
// On désactive des extension car à cause de l'optimisation, des particules ne
// peuvent pas être créé.

func TestAdd2(t*testing.T){

  config.Get("../configtest.json")
  config.General.InitNumParticles = 0
  config.General.SpawnRate = 0.5
  config.General.LimiteActive = false
  config.General.DureevieActive = false
  S := NewSystem()

  for i:=0; i< 60;i++{
    S.Update()

  }

  if len(S.Content) != 30{
    t.Fail()
  }
}

// Cette fonction test si Update génére 1 particule un appel sur deux de
//avec la méthode Add et même si il y a 10 particules de généré de base.
// On désactive des extension car à cause de l'optimisation, des particules ne
// peuvent pas être créé.

func TestAdd3(t*testing.T){

  config.Get("../configtest.json")
  config.General.InitNumParticles = 10
  config.General.SpawnRate = 0.5
  config.General.LimiteActive = false
  config.General.DureevieActive = false
  S := NewSystem()

  for i:=0; i< 60;i++{
    S.Update()
  }

  if len(S.Content) != 40{
    t.Fail()
  }
}

// Cette fonction test si Update génére 1 particule au bout de 60 appels soit
// au bout de 1 seconde avec la méthode Add.
// Le nombre inscrit pour config.General.SpawnRate est
// un nombre arrondie de la fraction 1/60 qui donnt 1 particule / seconde.
// On désactive des extension car à cause de l'optimisation, des particules ne
// peuvent pas être créé.

func TestAdd4(t*testing.T){

  config.Get("../configtest.json")
  config.General.InitNumParticles = 0
  config.General.SpawnRate = 0.0166666667
  config.General.LimiteActive = false
  config.General.DureevieActive = false
  S := NewSystem()

  for i:=0; i< 60;i++{
    S.Update()
  }

  if len(S.Content) != 1{
    t.Fail()
  }
}

// Cette fonction test si les positions sont bien modifiées lors des différents
// appels de la fonction Updapte, ça permet de voir si les particules ont bien
// une vitesse et donc qu'elles se déplacent.

func TestVitesse(t*testing.T){

  config.Get("../configtest.json")
  config.General.InitNumParticles = 1
  config.General.LimiteActive= false
  config.General.DureevieActive = false
  S:= NewSystem()


      S.Update()
      S.Update()

  for i:=0; i<10;i++{
    v1 := S.Content[0].PositionX
    v2 := S.Content[0].PositionY
    S.Update()
    if v1 == S.Content[0].PositionX || v2 == S.Content[0].PositionY{
      t.Fail()
    }
  }
}


// Cette fonction test si quand les particules ont un positionnement aléatoire
// elles ne soient pas générées en dehors de l'écran défini.

func TestAleatoirePosition_Ecran(t*testing.T){
  config.Get("../configtest.json")
  config.General.InitNumParticles = 1
  config.General.RandomSpawn = true
  config.General.WindowSizeX = 800
  config.General.WindowSizeY = 800

  for i := 0; i < 60 ; i++ {
    S:= NewSystem()

    if S.Content[0].PositionX < 0 || S.Content[0].PositionX > float64(config.General.WindowSizeX)||S.Content[0].PositionY < 0 || S.Content[0].PositionY > float64(config.General.WindowSizeY){
      t.Fail()
      }
    }
}



// Cette fonction test si pour plusieurs lancement du programme où on veut que
//les particules ont une position fixe elles ont bien à chaque fois le
// positionnement qu'on leur a fixé même suite à plusieurs lancement du
// programme NewSystem.


func TestFixePosition(t*testing.T){

  config.Get("../configtest.json")
  config.General.InitNumParticles = 1
  config.General.RandomSpawn = false
  config.General.SpawnX = 400
  config.General.SpawnX = 400

  for i:=0; i < 60 ; i++{
      S:=NewSystem()
        if S.Content[0].PositionX != float64(config.General.SpawnX) || S.Content[0].PositionY != float64(config.General.SpawnY){
          t.Fail()
        }
      }
    }

/*Cette fonction test si quand une particule arrive à 0 de Durée de vie, elle arrête
bien de bouger à partir de la méthode Vie*/

func TestMort(t *testing.T){

  config.Get("../configtest.json")
  config.General.InitNumParticles = 1
  config.General.DureevieActive = true
  config.General.Dureevie = 60
  config.General.LimiteActive = false
  config.General.SpawnRate = 0

  S :=NewSystem()

  for i:=0; i < 60 ;i++{
    S.Content[0].Vie()
  }
  posX := S.Content[0].PositionX
  posY := S.Content[0].PositionY

  S.Content[0].PositionUpdate()

  if S.Content[0].PositionX != posX || S.Content[0].PositionY != posY{
    t.Fail()
  }
}

/*Cette fonction test l'extension gravité, je désactive l'extension Limitefixe
pour que la particule puisse continuer à se mettre à jour malgré qu'elle soit en
dehors de l'écran. */

func TestGravite(t *testing.T){

  config.Get("../configtest.json")
  config.General.InitNumParticles = 1
  config.General.GraviteActive = true
  config.General.Gravite = 5
  config.General.LimiteActive = false
  config.General.DureevieActive = false

  S := NewSystem()
  S.Content[0].SpeedY = 0

  for i:=0; i<60;i++{
    posY := S.Content[0].PositionY
    S.Content[0].Gravitee()


    if posY == S.Content[0].PositionY{
      t.Fail()
    }
  }
}

/*Cette fonction test l'extension Limite qui permet d'arréter de mettre à jour
et d'afficher une particule lorsqu'elle est en dehors de l'écran  à une certaine
longueur. */

func TestLimite(t *testing.T){

  config.Get("../configtest.json")
  config.General.InitNumParticles = 1
  config.General.LimiteActive = true
  config.General.LimiteX = 50
  config.General.LimiteY = 50
  config.General.Rebondis = false
  config.General.GraviteActive = false

  S :=NewSystem()

  for S.Content[0].PositionX < float64(config.General.WindowSizeX)+ float64(config.General.LimiteX) && S.Content[0].PositionX > 0 - float64(config.General.LimiteX) && S.Content[0].PositionY < float64(config.General.WindowSizeY)+ float64(config.General.LimiteY) && S.Content[0].PositionY > 0 - float64(config.General.LimiteX)  {
    S.Update()
  }

  S.Update()

  particulebefore := S.Content[0]
  S.Update()
  if particulebefore.PositionX != S.Content[0].PositionX{
    t.Fail()
  }

  if particulebefore.PositionY != S.Content[0].PositionY{
    t.Fail()
  }

}

/*Cette fonction test l'extension Rebondis qui permet aux particules de rebondir
contre les rebords de l'écran et donc de ne pas sortir de celui-ci.*/

func TestRebondis(t *testing.T){

  config.Get("../configtest.json")
  config.General.InitNumParticles = 5
  config.General.Rebondis = true
  config.General.ChangementActive = false
  config.General.GraviteActive = false
  config.General.DureevieActive = false


  S :=NewSystem()

  for i:=0; i < 6000; i++{
    for j:=0; j < len(S.Content); j++{
      S.Update()

      if S.Content[j].PositionX > float64(config.General.WindowSizeX) ||  S.Content[j].PositionY > float64(config.General.WindowSizeY) || S.Content[j].PositionX < 0 || S.Content[j].PositionY < 0{
        S.Update()
      }

      if S.Content[j].PositionX > float64(config.General.WindowSizeX) ||  S.Content[j].PositionY > float64(config.General.WindowSizeY) || S.Content[j].PositionX < 0 || S.Content[j].PositionY < 0{
        t.Fail()
      }
    }
  }
}

/*Cette fonction test une partie de l'extension Changement, elle test ici
l'évolution de couleurs des particules.*/

func TestChangement(t *testing.T){

  config.Get("../configtest.json")
  config.General.InitNumParticles = 10
  config.General.ChangementActive = true

  S:=NewSystem()

  for i:=0; i < 6000; i++{
    for j:=0; j < len(S.Content); j++{
      red := S.Content[j].ColorRed
      green := S.Content[j].ColorGreen
      blue := S.Content[j].ColorBlue
      S.Content[j].Changement()
      if S.Content[j].ColorRed == red || S.Content[j].ColorGreen == green || S.Content[j].ColorBlue == blue{
        t.Fail()
      }
    }
  }
}

/*Cette fonction test une partie de l'extension Changement, elle test ici
si les particles changent bien de taille au fur du temps.*/

func TestChangement2(t *testing.T){

  config.Get("../configtest.json")
  config.General.InitNumParticles = 10
  config.General.ChangementActive = true

  S:=NewSystem()

  for i:=0; i < 6000; i++{
    for j:=0; j < len(S.Content); j++{
      scaleX := S.Content[j].ScaleX
      scaleY := S.Content[j].ScaleY

      S.Content[j].Changement()
      if scaleX == S.Content[j].ScaleX || scaleY == S.Content[j].ScaleY{
        t.Fail()
      }
    }
  }
}

/*Cette fonction test une partie de l'extension Changement, elle test ici
si les particules ont bien la rotation qu'on leur donne.*/

func TestChangement3(t *testing.T){

  config.Get("../configtest.json")
  config.General.InitNumParticles = 10
  config.General.ChangementActive = true
  config.General.Rotation = 0.5

  S:=NewSystem()

  for i:=0; i < 6000; i++{
    for j:=0; j < len(S.Content); j++{
      Rotation := S.Content[j].Rotation

      S.Content[j].Changement()
      if S.Content[j].Rotation ==   Rotation{
        t.Fail()
      }
    }
  }
}
